/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.bonita.parsing.binding;

import java.util.Set;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.bonita.pvm.internal.xml.Parse;
import org.ow2.bonita.pvm.internal.xml.Parser;
import org.ow2.bonita.facade.def.majorElement.ActivityDefinition;
import org.ow2.bonita.facade.def.majorElement.ActivitySetDefinition;
import org.ow2.bonita.facade.def.majorElement.TransitionDefinition;
import org.ow2.bonita.facade.def.majorElement.impl.ActivitySetDefinitionImpl;
import org.ow2.bonita.facade.uuid.ActivitySetDefinitionUUID;
import org.ow2.bonita.facade.uuid.PackageDefinitionUUID;
import org.ow2.bonita.facade.uuid.ProcessDefinitionUUID;
import org.ow2.bonita.services.util.ServiceEnvTool;
import org.w3c.dom.Element;


public class ActivitySetBinding extends MajorElementBinding {

  public ActivitySetBinding() {
    super("ActivitySet");
  }

  private static final Logger LOG = Logger.getLogger(ActivitySetBinding.class.getName());

  public Object parse(final Element activitySetElement, final Parse parse, final Parser parser) {
    if (LOG.isLoggable(Level.FINE)) {
      LOG.fine("parsing element = " + activitySetElement);
    }


    final String id = getId(activitySetElement);
    final String name = getName(activitySetElement);

    final ActivitySetDefinitionUUID recordUUID =
      ServiceEnvTool.getUUIDGenerator().getActivitySetDefinitionUUID(
          getObject(ProcessDefinitionUUID.class, parse),
          id);
    parse.pushObject(recordUUID);

    final Set<ActivityDefinition> activities =
      parseXpdlMajorElementList(activitySetElement, "Activities", "Activity", parse, parser);
    final Set<TransitionDefinition> transitions = parseXpdlMajorElementList(activitySetElement,
        "Transitions", "Transition", parse, parser);

    final ActivitySetDefinition activitySet = new ActivitySetDefinitionImpl(recordUUID, id,
        getObject(PackageDefinitionUUID.class, parse),
        getObject(ProcessDefinitionUUID.class, parse), name, activities, transitions);

    parse.popObject();
    return activitySet;
  }

}


